require "class"
local InvSlot = require "widgets/invslot"
local TileBG = require "widgets/tilebg"
local Image = require "widgets/image"
local Widget = require "widgets/widget"
local EquipSlot = require "widgets/equipslot"
local ItemTile = require "widgets/itemtile"
local Text = require "widgets/text"

local HUD_ATLAS = "images/hud.xml"

local Inv = Class(Widget, function(self, owner)
    Widget._ctor(self, "Inventory")
    self.owner = owner

	self.base_scale = .6
	self.selected_scale = .8

    self:SetScale(self.base_scale)
    self:SetPosition(0,-16,0)

    self.inv = {}
    self.equip = {}

    self.equipslotinfo =
    {
	}

    self.bg = self:AddChild(Image(HUD_ATLAS, "inventory_bg.tex"))
    self.bg:SetVRegPoint(ANCHOR_BOTTOM)
    self.bg:SetScale(1.21,1,1)

    self.hovertile = nil

	self.cursor = self:AddChild(Image( HUD_ATLAS, "slot_select.tex"))
	self.cursor:Hide()
	
	self.repeat_time = .2

	--this is for the keyboard / controller inventory controls
	self.actionstring = self:AddChild(Widget("actionstring"))
	self.actionstring:SetScale(2)

	self.actionstringtitle = self.actionstring:AddChild(Text(TITLEFONT, 40))
	self.actionstringtitle:SetRegionSize(250, 50)
	self.actionstringtitle:SetVAlign(ANCHOR_BOTTOM)
	self.actionstringtitle:SetPosition(0,125)
	self.actionstringtitle:SetColour(1,0,1,1)

	self.actionstringbody = self.actionstring:AddChild(Text(BODYTEXTFONT, 25))
	self.actionstringbody:SetRegionSize(200, 100)
	self.actionstringbody:EnableWordWrap(true)
	self.actionstringbody:SetPosition(0,50)
    self.actionstringbody:SetVAlign(ANCHOR_TOP)
    self.actionstringbody:SetHAlign(ANCHOR_LEFT)
	self.actionstring:Hide()


	--default equip slots
	self:AddEquipSlot(EQUIPSLOTS.HANDS, HUD_ATLAS, "equip_slot.tex")
	self:AddEquipSlot(EQUIPSLOTS.BODY, HUD_ATLAS, "equip_slot_body.tex")
	self:AddEquipSlot(EQUIPSLOTS.HEAD, HUD_ATLAS, "equip_slot_head.tex")
	self:AddEquipSlot(EQUIPSLOTS.NECK, HUD_ATLAS, "equip_slot_body.tex")
	self:AddEquipSlot(EQUIPSLOTS.BACK, HUD_ATLAS, "equip_slot_body.tex")


    self.inst:ListenForEvent("builditem", function(inst, data) self:OnBuild() end, self.owner)
    self.inst:ListenForEvent("itemget", function(inst, data) self:OnItemGet(data.item, data.slot, data.src_pos) end, self.owner)
    self.inst:ListenForEvent("equip", function(inst, data) self:OnItemEquip(data.item, data.eslot) end, self.owner)
    self.inst:ListenForEvent("unequip", function(inst, data) self:OnItemUnequip(data.item, data.eslot) end, self.owner)
    self.inst:ListenForEvent("newactiveitem", function(inst, data) self:OnNewActiveItem(data.item) end, self.owner)
    self.inst:ListenForEvent("itemlose", function(inst, data) self:OnItemLose(data.slot) end, self.owner)
end)


function Inv:AddEquipSlot(slot, atlas, image, sortkey)
	sortkey = sortkey or #self.equipslotinfo
	table.insert(self.equipslotinfo, {slot = slot, atlas = atlas, image = image, sortkey = sortkey})
	table.sort(self.equipslotinfo, function(a,b) return a.sortkey < b.sortkey end)
	self.rebuild_pending = true
end

function Inv:Rebuild()

    for k,v in pairs(self.inv) do
    	v:Kill()
    end

    for k,v in pairs(self.equip) do
    	v:Kill()
    end

    self.inv = {}
    self.equip = {}

    local num_slots = 15
    local num_equip = #self.equipslotinfo

    local W = 64
    local SEP = 12
    local INTERSEP = 28
    local y = 132/2
    
    local num_intersep = math.floor(num_slots / 5) + 1 
    
    local total_w = (num_slots + num_equip)*(W) + (num_slots + num_equip - 2 - num_intersep) *(SEP) + INTERSEP*num_intersep
    for k, v in ipairs(self.equipslotinfo) do

        local slot = EquipSlot(v.slot, v.atlas, v.image, self.owner)
        self.equip[v.slot] = self:AddChild(slot)
        local x = -total_w/2 + (num_slots)*(W)+num_intersep*(INTERSEP - SEP) + (num_slots-1)*SEP + INTERSEP + W*(k-1) + SEP*(k-1)
        slot:SetPosition(x,y,0)
    end    

    for k = 1,num_slots do
        local slot = InvSlot(k, HUD_ATLAS, "inv_slot.tex", self.owner, self.owner.components.inventory)
        self.inv[k] = self:AddChild(slot)
        local interseps = math.floor((k-1) / 5)
        local x = -total_w/2 + W/2 + interseps*(INTERSEP - SEP) + (k-1)*W + (k-1)*SEP
        slot:SetPosition(x,y,0)
    end

	self.actionstring:MoveToFront()
	self.cursor:MoveToFront()

	self.rebuild_pending = false
end


function Inv:Update(dt)

	if self.rebuild_pending == true then
		self:Rebuild()
		self:Refresh()
	end


	if not self.open then return end
	
	if self.repeat_time > 0 then
		self.repeat_time = self.repeat_time - dt
	end
	
	if self.repeat_time <= 0 then
		if TheInput:IsControlPressed(CONTROL_FOCUS_LEFT) then
			self:CursorLeft()
		elseif TheInput:IsControlPressed(CONTROL_FOCUS_RIGHT) then
			self:CursorRight()
		elseif TheInput:IsControlPressed(CONTROL_FOCUS_UP) then
			self:CursorUp()
		elseif TheInput:IsControlPressed(CONTROL_FOCUS_DOWN) then
			self:CursorDown()
		else
			self.repeat_time = 0
			self.reps = 0
			return
		end
		
		self.reps = self.reps and (self.reps + 1) or 1
		
		if self.reps == 1 then
			self.repeat_time = 6/30
		elseif self.reps < 2 then
			self.repeat_time = 4/30
		else
			self.repeat_time = 3/30
		end
		
	end	
end


function Inv:OffsetCursor(offset, val, minval, maxval, slot_is_valid_fn)
	if val == nil then
		val = minval
	else
		
		local idx = val
		local start_idx = idx

		repeat 
			idx = idx + offset
			
			if idx < minval then idx = maxval end
			if idx > maxval then idx = minval end

			if slot_is_valid_fn(idx) then 
				val = idx
				break
			end

		until start_idx == idx
	end
	
	return val
end



function Inv:CursorLeft()
	self:SelectSlot( self:OffsetCursor(-1, self.active_slot, 1, #self.inv, function(idx) return self.owner.components.inventory:GetItemInSlot(idx) or self.owner.components.inventory:GetActiveItem() end ) )
end

function Inv:CursorRight()
	self:SelectSlot( self:OffsetCursor(1, self.active_slot, 1, #self.inv, function(idx) return self.owner.components.inventory:GetItemInSlot(idx) or self.owner.components.inventory:GetActiveItem() end ) )
end

function Inv:CursorUp()

end

function Inv:CursorDown()

end


function Inv:OnControl(control, down)
	if Inv._base.OnControl(self, control, down) then return true end
	
	if self.open then
		if not down then 
			if control == CONTROL_ACCEPT then
				self.inv[self.active_slot]:Click()
			end
		end
	end
	
end


function Inv:OpenControllerInventory()
	if not self.open then
		if not self.active_slot then
			self:SelectSlot(1)
		end
		self:UpdateCursor()
		self.open = true
		self:ScaleTo(self.base_scale,self.selected_scale,.2)
		TheFrontEnd:LockFocus(true)
		self:SetFocus()
	end
end

function Inv:CloseControllerInventory()
	if self.open then
		self.open = false
		self.cursor:Hide()
		self.actionstring:Hide()

		self.owner.components.inventory:ReturnActiveItem()

		if self.active_slot then
			self.inv[self.active_slot]:DeHighlight()
		end
		
		self:ScaleTo(self.selected_scale, self.base_scale,.1)
		TheFrontEnd:LockFocus(false)
	end
end


function Inv:UpdateActionText()
	local item = self.owner.components.inventory:GetActiveItem() or (self.active_slot and  self.owner.components.inventory:GetItemInSlot(self.active_slot))
	if item and self.active_slot then
		self.actionstring:Show()
		self.actionstring:SetPosition(self.inv[self.active_slot]:GetPosition() + Vector3(0,100,0))
		self.actionstringtitle:SetString(item.name)
	    self.actionstringbody:SetString("body text showing actions!")
	else
		self.actionstring:Hide()
	end
end

function Inv:SelectSlot(idx)
	if self.active_slot and self.active_slot ~= idx then
		self.inv[self.active_slot]:DeHighlight()
	end
	self.active_slot = idx

	self:UpdateCursor()
end

function Inv:UpdateCursor()

	if self.active_slot and self.hovertile then
		self.hovertile:SetPosition(self.inv[self.active_slot]:GetPosition() )
	end

	if self.active_slot then

		self.cursor:Show()
		self.inv[self.active_slot]:AddChild(self.cursor)
		
		self.inv[self.active_slot]:Highlight()
		if self.inv[self.active_slot].tile then
			print (self.inv[self.active_slot].tile:GetDescriptionString())
		end
	else
		self.cursor:Hide()
	end

	self:UpdateActionText()

end

function Inv:Refresh()
	
	for k,v in pairs(self.inv) do
		v:SetTile(nil)
	end

	for k,v in pairs(self.equip) do
		v:SetTile(nil)
	end

	for k,v in pairs(self.owner.components.inventory.itemslots) do
		if v then
			local tile = ItemTile(v, self)
			self.inv[k]:SetTile(tile)
		end
	end

	for k,v in pairs(self.owner.components.inventory.equipslots) do
		if v then
			local tile = ItemTile(v, self)
			self.equip[k]:SetTile(tile)
		end
	end
	
	self:OnNewActiveItem(self.owner.components.inventory.activeitem)

end


function Inv:Cancel()
    local active_item = self.owner.components.inventory:GetActiveItem()
    if active_item then
        self.owner.components.inventory:ReturnActiveItem()
    end
end

function Inv:OnItemLose(slot)
	if slot then
		self.inv[slot]:SetTile(nil)
	end
end

function Inv:OnBuild()
    if self.hovertile then
        self.hovertile:ScaleTo(3, 1, .5)
    end
end

function Inv:OnNewActiveItem(item)
    if self.hovertile then
        self.hovertile:Kill()
    end

    if item and self.owner.HUD.controls then
    	
    	if self.open then
        	self.hovertile = self:AddChild(ItemTile(item, self))
        	self:UpdateCursor()
        	self.hovertile:SetScale(1.8)
        else
        	self.hovertile = self.owner.HUD.controls.mousefollow:AddChild(ItemTile(item, self))
        end

        self.hovertile:StartDrag()
    end

end

function Inv:OnItemGet(item, slot, source_pos)
    if slot and self.inv[slot] then
		local tile = ItemTile(item, self)
        self.inv[slot]:SetTile(tile)
        tile:Hide()

        if source_pos then
			local dest_pos = self.inv[slot]:GetWorldPosition()
			local im = Image(item.components.inventoryitem:GetAtlas(), item.components.inventoryitem:GetImage())
			im:MoveTo(source_pos, dest_pos, .3, function() tile:Show() tile:ScaleTo(2, 1, .25) im:Kill() end)
        else
			tile:Show() 
			--tile:ScaleTo(2, 1, .25)
        end
        
	end
end

function Inv:OnItemEquip(item, slot)
    self.equip[slot]:SetTile(ItemTile(item, self))
end

function Inv:OnItemUnequip(item, slot)
    if slot and self.equip[slot] then
		self.equip[slot]:SetTile(nil)
	end
end

return Inv