--[[

This mod shows how to load custom prefabs and assets so that they are usable
by the game.

--]]
-- The Assets list here in modmain.lua is used to alert the game to any textures
-- that you need to load that aren't part of a prefab. (The assets of a prefab
-- are listed within the prefab. Also use this to overwrite in-game textures.
Assets = {
	
	Asset("ANIM", MODROOT.."anim/leekbaton.zip"),
	Asset("ANIM", MODROOT.."anim/swap_leekbaton.zip"),
	Asset("IMAGE", MODROOT.."inventoryimages/leekbaton.tex"),
	Asset("ATLAS", MODROOT.."inventoryimages/leekbaton.xml"),
	
	Asset("ANIM", MODROOT.."anim/mikuhg.zip"),
	Asset("ANIM", MODROOT.."anim/swap_mikuhg.zip"),
	Asset("IMAGE", MODROOT.."inventoryimages/mikuhg.tex"),
	Asset("ATLAS", MODROOT.."inventoryimages/mikuhg.xml"),
	
	Asset("ANIM", MODROOT.."anim/emptyhg.zip"),
	Asset("IMAGE", MODROOT.."inventoryimages/emptyhg.tex"),
	Asset("ATLAS", MODROOT.."inventoryimages/emptyhg.xml"),
	
	Asset("ANIM", MODROOT.."anim/rhammo.zip"),
	Asset("IMAGE", MODROOT.."inventoryimages/rhammo.tex"),
	Asset("ATLAS", MODROOT.."inventoryimages/rhammo.xml"),
	
	Asset("ANIM", MODROOT.."anim/srhammo.zip"),
	Asset("IMAGE", MODROOT.."inventoryimages/srhammo.tex"),
	Asset("ATLAS", MODROOT.."inventoryimages/srhammo.xml"),
	
	Asset("ANIM", MODROOT.."anim/mikuhgs.zip"),
	Asset("ANIM", MODROOT.."anim/swap_mikuhgs.zip"),
	Asset("IMAGE", MODROOT.."inventoryimages/mikuhgs.tex"),
	Asset("ATLAS", MODROOT.."inventoryimages/mikuhgs.xml"),
	
	Asset("ANIM", MODROOT.."anim/bullet.zip"),
	
	Asset("IMAGE", MODROOT.."images/mikutab.tex"),
	Asset("ATLAS", MODROOT.."images/mikutab.xml"),
	
}



-- The PrefabFiles list is the list of all the files in your <modname>/scripts/prefabs folder
-- that you want to load prefabs from
PrefabFiles = {

	"leekbaton",
	"mikuhg",
	"emptyhg",
	"rhammo",
	"srhammo",
	"mikuhgs",
	"bullet",
	
}

local Ingredient = GLOBAL.Ingredient
local RECIPETABS = GLOBAL.RECIPETABS
local STRINGS = GLOBAL.STRINGS


-- Import things we like into our mod's own global scope, so we don't have 
-- to type "GLOBAL." every time want to use them.
SpawnPrefab = GLOBAL.SpawnPrefab

-- function SimInit(player)
	-- print("spawning item")
	-- local prefab = SpawnPrefab("srhammo")
	-- player.components.inventory:GiveItem(prefab)
-- end


-- thanks to no_signal for right-click to reload

local RELOAD = GLOBAL.Action(5)

RELOAD.fn = function(act)
	local item = act.doer.components.inventory:FindItem(function(item)
		        return item.prefab == "emptyhg"
		    end)
	act.doer.components.inventory:RemoveItem(item)
	act.doer.components.inventory:RemoveItem(act.invobject)
	local gun = GLOBAL.SpawnPrefab("mikuhg")
	act.doer.components.inventory:GiveItem(gun)
end

RELOAD.str = "Reload"
RELOAD.id = "RELOAD"
GLOBAL.ACTIONS.RELOAD = RELOAD
GLOBAL.STRINGS.ACTIONS.RELOAD = RELOAD.str
-- use "dolongaction" if you want a longer reload time
local actionHandler = GLOBAL.ActionHandler(RELOAD, "doshortaction")

local SGwilson = GLOBAL.require("stategraphs/SGwilson")
SGwilson.actionhandlers[RELOAD] = actionHandler
GLOBAL.package.loaded["stategraphs/SGwilson"] = SGwilson


local RELOADS = GLOBAL.Action(5)

RELOADS.fn = function(act)
	local item = act.doer.components.inventory:FindItem(function(item)
		        return item.prefab == "emptyhg"
		    end)
	act.doer.components.inventory:RemoveItem(item)
	act.doer.components.inventory:RemoveItem(act.invobject)
	local gun = GLOBAL.SpawnPrefab("mikuhgs")
	act.doer.components.inventory:GiveItem(gun)
end

RELOADS.str = "Reload Slow Ammo"
RELOADS.id = "RELOADS"
GLOBAL.ACTIONS.RELOADS = RELOADS
GLOBAL.STRINGS.ACTIONS.RELOADS = RELOADS.str
-- use "dolongaction" if you want a longer reload time
local actionHandler = GLOBAL.ActionHandler(RELOADS, "doshortaction")

local SGwilson = GLOBAL.require("stategraphs/SGwilson")
SGwilson.actionhandlers[RELOADS] = actionHandler
GLOBAL.package.loaded["stategraphs/SGwilson"] = SGwilson


GLOBAL.STRINGS.RECIPE_DESC.RHAMMO = "A box of 9mm rounds."
GLOBAL.STRINGS.RECIPE_DESC.SRHAMMO = "A Nation's secret technology."

STRINGS.TABS.MIKUTABS = "Himitsu Keisatsu"
count = 1
for _ in pairs(RECIPETABS) do count = count + 1 end
RECIPETABS["MIKUTABS"] = {str = STRINGS.TABS.MIKUTABS, sort = count, icon = "mikutab.tex", icon_atlas = "images/mikutab.xml"}

function handgun()
local rhammo = GLOBAL.Recipe("rhammo", { Ingredient("slurtle_shellpieces", 3), Ingredient("flint", 6), Ingredient("gunpowder", 2) }, RECIPETABS.MIKUTABS, {SCIENCE = 0})
local srhammo = GLOBAL.Recipe("srhammo", { Ingredient("rhammo", 1), Ingredient("silk", 12), Ingredient("spidergland", 6) }, RECIPETABS.MIKUTABS, {SCIENCE = 0})

rhammo.atlas = "inventoryimages/rhammo.xml"
srhammo.atlas = "inventoryimages/srhammo.xml"

end

     -- local myrecipe = GLOBAL.Recipe( "myprefab", { Ingredient("wall_wood_item", 10), Ingredient("turf_carpetfloor", 10), Ingredient("slurtleslime", 10) }, RECIPETABS.TOWN, 2)
-- myrecipe.atlas = "images/inventoryimages/myprefabatlas.xml" -- has 'myprefab.tex' specified inside it.

AddGamePostInit(handgun)



AddSimPostInit(SimInit )
