require "prefabutil"

function MakeMechType(data)

	local assets =
	{
		Asset("ANIM", "anim/mech.zip"),
		Asset("ANIM", "anim/mech_"..data.name..".zip"),
		--Asset("IMAGE", "images/inventoryimages/mech_"..data.name.."_item.tex"),
		Asset("ATLAS", "images/inventoryimages/mech_"..data.name.."_item.xml"),
	}

	local function onhammered(inst, worker)

		if data.maxloots and data.loot then
			local num_loots = math.max(1, math.floor(data.maxloots*inst.components.health:GetPercent()))
			for k = 1, num_loots do
				inst.components.lootdropper:SpawnLootPrefab(data.loot)
			end
		end		
		
		SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
		
		if data.destroysound then
			inst.SoundEmitter:PlaySound(data.destroysound)		
		end
		
		inst:Remove()
	end
	
	local function test_wall(inst, pt)
		return true
	end
	
	local function makeobstacle(inst)
	
	    inst.Physics:ClearCollisionMask()
		inst.Physics:CollidesWith(COLLISION.WORLD)
		inst.Physics:CollidesWith(COLLISION.ITEMS)
		inst.Physics:CollidesWith(COLLISION.CHARACTERS)

	    local ground = GetWorld()
	    if ground then
	    	local pt = Point(inst.Transform:GetWorldPosition())
			--print("    at: ", pt)
	    	ground.Pathfinder:AddWall(pt.x, pt.y, pt.z)
	    end
	end
	
	local function clearobstacle(inst)
	    inst.Physics:ClearCollisionMask()
		inst.Physics:CollidesWith(COLLISION.WORLD)
		inst.Physics:CollidesWith(COLLISION.ITEMS)
	    local ground = GetWorld()
	    if ground then
	    	local pt = Point(inst.Transform:GetWorldPosition())
	    	--print("    at: ", pt)
	    	ground.Pathfinder:RemoveWall(pt.x, pt.y, pt.z)
	    end
	end
		
	local function onhealthchange(inst, old_percent, new_percent)
		
		if old_percent <= 0 and new_percent > 0 then makeobstacle(inst) end
		if old_percent > 0 and new_percent <= 0 then clearobstacle(inst) end

		local anim_to_play = nil
		if new_percent <= 0 then
			anim_to_play = "0"
		elseif new_percent <= .4 then
			anim_to_play = "1_4"
		elseif new_percent <= .5 then
			anim_to_play = "1_2"
		elseif new_percent <= .9 then
			anim_to_play = "3_4"
		else
			anim_to_play = "1"
		end

		if old_percent > new_percent and new_percent > 0 then
		
			inst.AnimState:PlayAnimation(anim_to_play.."_hit")		
			inst.AnimState:PushAnimation(anim_to_play)		
		else
			inst.AnimState:PlayAnimation(anim_to_play)		
		end
	end
	
	local function turnon(inst)
	 local name = nil
	 if inst.entity:HasTag("grass") then 
	 name = "straw"
	 elseif inst.entity:HasTag("wood") then 
	 name = "wood"  inst.Transform:SetScale(1, 1, 1)
	 elseif inst.entity:HasTag("stone") then
	 name = "stone"
	 end
	   local var = inst.components.health:GetPercent()
	   if var <= 0 then
	   inst.SoundEmitter:PlaySound("dontstarve/common/destroy_"..name)	
	   inst.components.machine.ison = false
	   inst.AnimState:PushAnimation("0")
	   elseif var <= .4 then
	   inst.components.machine.ison = true
	   inst.AnimState:PlayAnimation("1_4")
	   inst.SoundEmitter:PlaySound("dontstarve/common/place_structure_"..name)
	   makeobstacle(inst)
	   elseif var <= .5 then
	   inst.components.machine.ison = true
	   inst.AnimState:PlayAnimation("1_2")
	   inst.SoundEmitter:PlaySound("dontstarve/common/place_structure_"..name)
	   makeobstacle(inst)
	   elseif var <= .9 then
	   inst.components.machine.ison = true
	   inst.AnimState:PlayAnimation("3_4")
	   inst.SoundEmitter:PlaySound("dontstarve/common/place_structure_"..name)
	   makeobstacle(inst)
	   else
	   inst.AnimState:PlayAnimation("1")
	   inst.SoundEmitter:PlaySound("dontstarve/common/place_structure_"..name)
	   inst.components.machine.ison = true
	   makeobstacle(inst)
	   end
	end
	
	local function turnoff(inst, data)
     local name = nil
	 if inst.entity:HasTag("grass") then 
	 name = "straw"
	 elseif inst.entity:HasTag("wood") then 
	 name = "wood"
	 elseif inst.entity:HasTag("stone") then
	 name = "stone"
	 end
	   inst.components.machine.ison = false
	   inst.SoundEmitter:PlaySound("dontstarve/common/destroy_"..name)
           if  name == "wood" then inst.Transform:SetScale(0.3, 1, 1) 		
	   else inst.AnimState:PlayAnimation("0") end
	   clearobstacle(inst)
	end
	
	local function ondeploywall(inst, pt, deployer)
		--inst.SoundEmitter:PlaySound("dontstarve/creatures/spider/spider_egg_sack")
		local mech = SpawnPrefab("mech_"..data.name) 
		if mech then 
			pt = Vector3(math.floor(pt.x)+.5, 0, math.floor(pt.z)+.5)
			mech.Physics:SetCollides(false)
			mech.Physics:Teleport(pt.x, pt.y, pt.z) 
			mech.Physics:SetCollides(true)
			inst.components.stackable:Get():Remove()

		    local ground = GetWorld()
		    if ground then
		    	ground.Pathfinder:AddWall(pt.x, pt.y, pt.z)
				turnon(mech)
		    end
		end 
		
	end

    local function onhit(inst)
		if data.destroysound then
			inst.SoundEmitter:PlaySound(data.destroysound)		
		end
	end

	local function onrepaired(inst)
		if data.buildsound then
			inst.SoundEmitter:PlaySound(data.buildsound)		
		end
		makeobstacle(inst)
	end
	    
	local function onload(inst, data)
		--print("walls - onload")
		makeobstacle(inst)
		turnon(inst)
		if inst.components.health:GetPercent() <= 0 then
			clearobstacle(inst)
		turnoff(inst)
		end
	end

	local function onremoveentity(inst)
		--print("walls - onremoveentity")
		clearobstacle(inst)
	end

    local function fn(Sim)
		local inst = CreateEntity()
		local trans = inst.entity:AddTransform()
		local anim = inst.entity:AddAnimState()
		inst.entity:AddSoundEmitter()

		inst:AddTag("wall")
		MakeObstaclePhysics(inst, .5)    

		anim:SetBank("wall")
		anim:SetBuild("mech_"..data.name)
	    anim:PlayAnimation("half", false)
	    
		--inst:AddComponent("inspectable")
		inst:AddComponent("lootdropper")
		
		for k,v in ipairs(data.tags) do
		    inst:AddTag(v)
		end
		
		
		inst:AddComponent("repairable")
        if data.name == "ruins" then
		    inst.components.repairable.repairmaterial = "thulecite"
        else
		    inst.components.repairable.repairmaterial = data.name
        end
		inst.components.repairable.onrepaired = onrepaired
		
		inst:AddComponent("combat")
		inst.components.combat.onhitfn = onhit
		
		inst:AddComponent("health")
		inst.components.health:SetMaxHealth(data.maxhealth)
		inst.components.health.currenthealth = data.maxhealth / 2
		inst.components.health.ondelta = onhealthchange
		inst.components.health.nofadeout = true
		inst:AddTag("noauradamage")
		
		inst:AddComponent("machine")
		inst.components.machine.turnonfn = turnon
        inst.components.machine.turnofffn = turnoff
        inst.components.machine.cooldowntime = 0
		inst.components.machine.caninteractfn = function() return not inst.components.health:IsDead() end
		
		inst:AddComponent("playerprox")
	    inst.components.playerprox:SetDist(3,4)
	    inst.components.playerprox.onnear = function()
		if inst.components.machine.ison then
           turnoff(inst)
		   end
	    end
	    inst.components.playerprox.onfar = function() turnon(inst) end
		
		if data.flammable then
			MakeLargeBurnable(inst)
			MakeLargePropagator(inst)
			inst.components.burnable.flammability = .5
			
			--lame!
			if data.name == "wood" then
				inst.components.propagator.flashpoint = 30+math.random()*10			
			end
		else
			inst.components.health.fire_damage_scale = 0
		end

		if data.buildsound then
			inst.SoundEmitter:PlaySound(data.buildsound)		
		end
		
		inst:AddComponent("workable")
		inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
		inst.components.workable:SetWorkLeft(3)
		inst.components.workable:SetOnFinishCallback(onhammered)
		inst.components.workable:SetOnWorkCallback(onhit) 
				
		
	    inst.OnLoad = onload
	    inst.OnRemoveEntity = onremoveentity
		
		MakeSnowCovered(inst)
		
		return inst
	end

	local function itemfn(Sim)

		local inst = CreateEntity()
		inst:AddTag("wallbuilder")
		
		inst.entity:AddTransform()
		inst.entity:AddAnimState()
		MakeInventoryPhysics(inst)
	    
		inst.AnimState:SetBank("wall")
		inst.AnimState:SetBuild("mech_"..data.name)
		inst.AnimState:PlayAnimation("idle")

		inst:AddComponent("stackable")
		inst.components.stackable.maxsize = TUNING.STACK_SIZE_MEDITEM
		
		inst:AddComponent("inspectable")
		inst:AddComponent("inventoryitem")
		inst.components.inventoryitem.atlasname = "images/inventoryimages/mech_"..data.name.."_item.xml"
		
		inst:AddComponent("repairer")
        if data.name == "ruins" then
		    inst.components.repairer.repairmaterial = "thulecite"
        else
		    inst.components.repairer.repairmaterial = data.name
        end

		inst.components.repairer.healthrepairvalue = data.maxhealth / 6
	    
		
		if data.flammable then
			MakeSmallBurnable(inst, TUNING.MED_BURNTIME)
			MakeSmallPropagator(inst)
			
			inst:AddComponent("fuel")
			inst.components.fuel.fuelvalue = TUNING.SMALL_FUEL
		end
		
		inst:AddComponent("deployable")
		inst.components.deployable.ondeploy = ondeploywall
		inst.components.deployable.test = test_wall
		inst.components.deployable.min_spacing = 0
		inst.components.deployable.placer = "mech_"..data.name.."_placer"
		
		return inst
	end
	
	

	


	return Prefab( "common/mech_"..data.name, fn, assets),
		   Prefab( "common/mech_"..data.name.."_item", itemfn, assets),
		   MakePlacer("common/mech_"..data.name.."_placer", "wall", "mech_"..data.name, "half", false, false, true) 
end

local mechprefabs = {}

--6 rock, 8 wood, 4 straw
local mechdata = {
			{name = "stone", tags={"stone"}, loot = "rocks", maxloots = 2, maxhealth=TUNING.STONEWALL_HEALTH, buildsound="dontstarve/common/place_structure_stone", destroysound="dontstarve/common/destroy_stone", repair = "wall_stone_item"},
			{name = "wood", tags={"wood"}, loot = "log", maxloots = 2, maxhealth=TUNING.WOODWALL_HEALTH, flammable = true, buildsound="dontstarve/common/place_structure_wood", destroysound="dontstarve/common/destroy_wood"},
			{name = "hay", tags={"grass"}, loot = "cutgrass", maxloots = 2, maxhealth=TUNING.HAYWALL_HEALTH, flammable = true, buildsound="dontstarve/common/place_structure_straw", destroysound="dontstarve/common/destroy_straw"}}

for k,v in pairs(mechdata) do
	local mech, item, placer = MakeMechType(v)
	table.insert(mechprefabs, mech)
	table.insert(mechprefabs, item)
	table.insert(mechprefabs, placer)
end


return unpack(mechprefabs)
